<?php

	/****************************************************************
	*@file formats.php
	*@package FFPROBE
	*@name Formats the output given by -formats to a ini format 
	*@author Actarus
	*@version V1.0
	*@note 
	*	V1.0 (24.02.2011) Working release
	******************************************************************/

	define('FFFORMATS_SECTIONS','File formats;Codecs;Bitstream filters;Supported file protocols;Frame size, frame rate abbreviations;Notes');
	

	function FFPROBE_get_php_formats(){
	//----------------------------------
		global $FFPROBE_formats;
		$FFPROBE_formats=FFProbe_show_formats();
		return $FFPROBE_formats;
	}
	
	function FFPROBE_get($SectionName){
	//-----------------------------
		global $FFPROBE_formats;
		$SectionNames=FFPROBE_get_SectionNames();
		if(in_array($SectionName,$SectionNames)) {
			if(is_array($FFPROBE_formats)) {
				if(!array_key_exists($SectionName,$FFPROBE_formats)) die("<br>Your FFPROBE is outdated, -formats has not the ".$SectionName." section required here, please consider using the SVN one.Do a cleanup using the cleanup.bat before.");
			}
			
			$section=$FFPROBE_formats[$SectionName];
			
			if(is_array($section)) {
				$section=array_shift($section);//FIXME: wonder why it's wrapped here
			}
			
			return $section;
		} else {
			die("FFPROBE_get: '$SectionName' is not a section name");
		}
	}	
		
	if(!function_exists("array_combine")) { //php4 support
	
		function array_combine($arr1, $arr2) {
		//-----------------------------------
			$out = array();
		   
			$arr1 = array_values($arr1);
			$arr2 = array_values($arr2);
		   
			foreach($arr1 as $key1 => $value1) {
				$out[(string)$value1] = $arr2[$key1];
			}
		   
			return $out;
		}
	}
	
	function FFMPEG_normalizeFlags($flags,$sep) {
	//-------------------------------------
		$chars = preg_split('//', $flags, -1, PREG_SPLIT_NO_EMPTY);
		return implode($sep,$chars);
	}
	
	function FFPROBE_get_SectionNames() {
	//-----------------------------------
		return explode(";",FFFORMATS_SECTIONS);
	}	
	
	class FFPROBE_Formats {
	 //@@@@@@@@@@@@@@@@@@@
	 
		var $content;
		var $SectionNames;
	 
	   //accept a filename or ini content as first parameter
		function FFPROBE_Formats($content, $SafeFile = false){ //__construct
		//==================================================
			$this->content=$content;
			$this->SectionNames=FFPROBE_get_SectionNames();
		}
		
		function makeINI() {
		//-------------------------------
			$content=$this->content;
			$content=$this->makeSections($content);
			$content=$this->makeSectionsITems($content);
			return $content;
		}
		
		function makeSections($out) {
		//---------------------------
			$SectionNames=$this->SectionNames;
			
			foreach($SectionNames as $SectionName) {
				$firstkey="\n";
				if($SectionName == 'Bitstream filters') $firstkey="\nfilters=";
				if($SectionName == 'Supported file protocols') $firstkey="\nprotos=";
				if($SectionName == 'Frame size, frame rate abbreviations') $firstkey="\nabbrevs=";
				$out=str_replace("$SectionName:\n","\n[$SectionName]$firstkey",$out);
			}
			
			$out=str_replace("Note,","[Notes]\ntext=Note,",$out);
			
			return $out;
		}
		
	
		function makeSectionFileformat($matches) {
		//------------------------------------
			$sep=':';
			$flags=FFMPEG_normalizeFlags($matches[1],$sep);
			$ext=trim($matches[2]);
			$desc=$matches[3];
			$format="\n$ext=$flags$sep$desc";
			return $format;
		}
		
		function makeSectionCodecs($matches) {
		//------------------------------------
			$sep=':';
			$flags=FFMPEG_normalizeFlags($matches[1],$sep);
			$ext=trim($matches[2]);
			$desc=$matches[3];
			$format="\n$ext=$flags$sep$desc";
			return $format;
		}
		
		function makeSectionFilters($matches) {
		//------------------------------------
			$filters=trim($matches[1]);
			$filters="filter=".str_replace(" ",":",$filters);
			return $filters;
		}
		
		function makeSectionProtocols($matches) {
		//------------------------------------
			$protos=trim($matches[1],':');
			$protos="protos=".str_replace(" ","",$protos);
			return $protos;
		}
		
		function makeSectionAbbreviations($matches) {
		//------------------------------------------
			$abbrevs=trim($matches[1]);
			$abbrevs="abbrevs=".str_replace(" ",":",$abbrevs);
			return $abbrevs;
		}
		
		function makeSectionNote($sep,$out) {
		//---------------------------------
			$notes=explode($sep,$out);
			if(count($notes)==2) {
				$section=$notes[0];
				$note=$notes[1];
				$lines=explode("\n",$note);
				$note='';
				foreach($lines as $no=>$line) {
					$note.="text$no=$line\n";
				}
				$out=$section.$note;
			}
			return $out;
		}
		
		function makeSection($section,$regexp,$from) {
		//---------------------------------------------
			if($section =="Note") {
				$from=$this->makeSectionNote($regexp,$from);
			}else {
				$from=preg_replace_callback($regexp,
						create_function(
							'$matches',
							'$classname = "'.get_class($this).'";'.
							'$funcname = "makeSection'.$section.'";'.
							'return call_user_func(array($classname,$funcname),$matches);'
						),
						$from
				);
			}
			return $from;
		}
		
		function makeSectionsItems($out) {
		//----------------------------
			//File formats: [DE] ext desc
			$out=$this->makeSection("Fileformat","/\n\s([DE ]{2})\s(\S{2,23})\s+(.*)/",$out);
	
			//Codecs: [DEVSDT ] ext desc
			$out=$this->makeSection("Codecs","/\s([DEAVST ]{7})\s(\S+)\s+(.*)/",$out);
		
			//Bitstream filters: filter1 filter2 ... filterN
			$out=$this->makeSection("Filters","/filters=(.*)/",$out);
			
			//Supported file protocols: proto1: proto2: ... protoN
			$out=$this->makeSection("Protocols","/protos=(.*)/",$out);
			
			//Frame size, frame rate abbreviations: abbrev1 abbrev2 ... abbrevN
			$out=$this->makeSection("Abbreviations","/abbrevs=(.*)/",$out);
			
			//Note contains a multiline text
			$out=$this->makeSection("Note","text=",$out);
			
			return $out;
		}
		
	}   

?>