<?php

	/****************************************************************
	*@file graph.php
	*@package FFPROBE
	*@name  Wrapper to generate graphs 
	*@internal A graph <type> plugin consists of a php file matching the name of the <type> graph 
	* it must contain the class FFprobe_<type>_graph , an example is given with bitrate graph.
	* all graph type plugins are automaticaly loaded and resides in ./graph/
	*@author Actarus
	*@requires PHP_GNUPlot.php >=2.0
	*@version V1.0
	*@note 
	*	V1.0 (28.02.2011) Working release with plugin feature
	******************************************************************/


	if(!defined("PHP_GNUPLOT_VERSION")) die("FFProbe graph requires php GNUPLOT!");
	
	define("FFPROBE_GRAPH_PLUGINS_DIR",fix_path(dirname(__FILE__).'/graph/'));
	define("FFPROBE_GRAPH_PLUGINS_LOAD_WARNINGS",0);
	define("FFPROBE_GRAPH_VERSION","1.0");
	define("FFPROBE_GRAPH_API",FFPROBE_GRAPH_VERSION);
	
	$FFprobe_graphs=array(); //Will hold the list of graphs types => graph class 
	
	//autoload graph type plugins from the given directory
	function FFPROBE_load_graph_plugins($dir) {
	//----------------------------------
		global $FFprobe_graphs;
		$handle=opendir($dir);
		$count=0;
		while ($file = readdir($handle)) 
		{
			if ($file=="." || $file=="..") continue;
			if (!is_dir($file)&&preg_match('/(\w+)\.php$/',$file,$plugin))
			{		
				$plugin=$plugin[1];
				$graph_plugin_class="FFprobe_{$plugin}_graph";
				$graph_plugin_file=FFPROBE_GRAPH_PLUGINS_DIR.$file;
				include($graph_plugin_file);
				if(class_exists($graph_plugin_class)) {
					$FFprobe_graphs[$plugin]=new $graph_plugin_class();//register plugin
				}else {
					if(FFPROBE_GRAPH_PLUGINS_LOAD_WARNINGS) echo "<br>Can find plugin class for $plugin graph in $graph_plugin_file";
				}
				$count++;
			}
		}
		closedir($handle);
		return $count;
	}
	
	//here we go...register graph type plugins found in $FFprobe_graphs;
	FFPROBE_load_graph_plugins(FFPROBE_GRAPH_PLUGINS_DIR);

	
	function array_key_filter($array,$callback,$param) {
	//-------------------------------------------------
		$array_filtered=array();
		
		if(is_string($callback)) {
			foreach($array as $id =>$array_values) {
				$array_filtered[]=call_user_func($callback,$array_values,$param);
			}
		}else { //Array classname, classfunc
			foreach($array as $id =>$array_values) {
				$array_filtered[]=call_user_func_array($callback,array($array_values,$param));
			}
		}
		
		return $array_filtered;
	}

	function ffprobegraph($type,$file) {
	//--------------------------------
		global $FFprobe_graphs;
		if(array_key_exists($type,$FFprobe_graphs)){
			//$FFprobe_graph=$create_function('$file','return new FFprobe_'.bitrate.'_graph($file)');
			$graph=$FFprobe_graphs[$type];
			$graph->init($file);
		}else {
			echo "<br>ffprobegraph: graph type '$type' not supported";
			$graph=new FFprobe_graph('','',0); //Dummy
		}
		return $graph;
	}
	
	class FFprobe_graph {
	///@@@@@@@@@@@@@@@@@@

		var $type;
		var $version;
		var $date;
		var $author;
		
		var $file; //media file audio or video
		var $data; 
		
		
		function FFprobe_graph($file,$type,$version,$author) {
		//------------------------------------------
			$this->type=$type;
			$this->version=$version;
			$this->date=date("F d Y", filemtime($file));
			$this->author=$author;
			$this->file='';
			$this->data=null;
		}
		
		
		
		function get_version() {
		//----------------------
			return $this->version;
		}
		
		function get_date() {
		//----------------------
			return $this->date;
		}
		
		function get_author() {
		//----------------------
			return $this->author;
		}
		
		function get_file() {
		//-------------------
			return $this->file;
		}
		
		function get_file_name() {
		//------------------------
			$file_chunks=explode(".",$this->file);
			return $file_chunks[0];
		}
		
		function get_file_ext() {
		//---------------------
			$file_chunks=explode(".",$this->file);
			return $file_chunks[1];
		}
		
		function get_graph_type() {
		//---------------------------------
			return $this->type;
		}
		
		function _init($file) {
		//--------------------
			$this->file=$file;
			if(!is_null(ffprobeinit())) {
				$ret=FFPROBE_show_all($file);
				if($ret["success"]) {
					$this->data=$ret["return"];
				}else {
					$graph_type=$this->type;
					$ret["error"].="<br>Cannot generate $graph_type data file from $file";
				}
			}
			return $ret;
		}
		
		/**
		*Two behaviors:
		*  $video_name,$video_ext[,$safe_file] to generate the data file from the video
		*  or
		*  $video_in to read back the generated data_file
		**/
		function generate_data_file() {
		//-------------------------------
			$args=func_get_args();
			$nargs=count($args);
			$safe_file=false;
			
			
			if($nargs>2) {
				$video_name=$args[0];
				$video_ext=$args[1];
				$ret=FFProbe_show($video_name,$video_ext,$safe_file);
			}else {
				if($nargs>1) $safe_file=$args[1];
				 //read data back, faster that regerating each times
				$video_in=$args[0];
				$ret=FFProbe_show($video_in);
			}
			
			return $ret;
		}
		

	
		function save_data_to_file($data_file,$data) {
		//--------------------------------------------
				$pg=new PGData();
				$pg->DataList=$data;
				$pg->dumpIntoFile($data_file); 
		}

		function save_data_file($data_type,$data_file,$data) {
		//-------------------------------------------------------------
			$graph_type=$this->get_graph_type();
			$data_file="{$data_file}_{$data_type}_{$graph_type}.dat";
			$this->save_data_to_file($data_file,$data);
		}

		function show_image_file($data_type,$data_file) {
		//----------------------------------
			$graph_type=$this->get_graph_type();
			$image_file="{$data_file}_{$data_type}_{$graph_type}.png";
			if(file_exists($image_file)) {
				echo "<script>document.write('<img src=\"$image_file\">');</script>";
			}
		}
		
		function get_data_type($type,$frame) {
		//---------------------------------------
			$type_frames=$frame["$type"];
			return array_key_filter($type_frames,array($this,'get_data'),$type);
		}
		
		function plot_data_file($data_type,$file='') {
		//--------------------------------------------
			$graph_type=$this->get_graph_type();
			$data_file="{$file}_{$data_type}_{$graph_type}.dat";
			if(!file_exists($data_file)) file_put_contents($data_file,'');
			$data = PGData::createFromFile($data_file);
			$this->plot_data($data,$data_type);
		}
		
		function show($type) {
		//--------------------
			if($this->get_file()) {
				$file_name=$this->get_file_name();
				$this->show_image_file($type,$file_name);
			}
		}
	
	}


?>