<?php

	/****************************************************************
	*@file bitrate.php
	*@package FFPROBE
	*@name  Generates bitrate grphs of video or audio files
	*@greetings I would like here to thanks 
	*	Michael Rampe for the original idea on http://forums.creativecow.net/thread/291/71
	*	Liu Yi (Eric) for making me discover Gnuplot syntax and power through PHPGnuPlot
	*@author Actarus
	*@requires PHP_GNUPlot.php >=2.0
	*@version V1.0
	*@note 
	*	V1.0 (28.02.2011) Working release
	******************************************************************/


	//color codes for each frame collection, seems to be B*255+G*256(G<<8)+R*65536(R<<16)
	define("IFRAMES",16711680);  //red   FF
	define("PFRAMES",65280);     //green FF
	define("BFRAMES",255);       //blue  FF


	class FFprobe_bitrate_graph extends FFprobe_graph {
	//@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

		function FFprobe_bitrate_graph() {
		//==================================
			$graph_type='bitrate';
			$graph_version='1.0';
			$graph_author='Actarus and Michael Rampe';
			$this->FFprobe_graph(__FILE__,$graph_type,$graph_version,$graph_author);
		}
		
		//Initializes and generates
		function init($file) {
		//--------------------
			$ret=$this->_init($file);
			if($ret["success"]) {
				$data=$this->data;
				if(is_array($data)) {
					if(array_key_exists("FRAME",$data)){
						$frame=$data["FRAME"];
						$this->plot_data_from_frame("audio",$frame);
						$this->plot_data_from_frame("video",$frame);
					}
				}
			}
		
		}
		
		//type is 'video' or 'audio'
		//frame is a container of <type> data frames 
		//show if true generats the html code to show the image of the plotted data
		function plot_data_from_frame($type,$frame,$show=false) {
		//-------------------------------------------------
			$file_name=$this->get_file_name();
			$file_ext=$this->get_file_ext();
			if(array_key_exists($type,$frame)) {
				$data=$this->get_data_type($type,$frame); 
				$this->save_data_file($type,$file_name,$data);
				$this->plot_data_file($type,$file_name);
				if($show) $this->show_image_file($type,$file_name);
			}
		}
		
	
		
		function getKeyFramesNumber($data) {
		//--------------------------------
			$stat=$data->getStats(0);
			return $stat[IFRAMES]; 
		}
		
		
		//$data is a PGData object holding datas
		function plot_data($data,$data_type) {
		//---------------------------------
		
			$yMin=$data->getMin(2);
			$yMax=intVal($data->getMax(2));

			$xMin=$data->getMin(1);
			$xMax=$data->getMax(1);

			$file_name=$this->get_file_name();
			$graph_type=$this->get_graph_type();
			
			$p = new GNUPlot("{$file_name}_{$data_type}_{$graph_type}.plt");
			
			$file=$this->get_file();
			$p->setTitle("$graph_type graph for the $data_type of $file");

			$mean=round($data->getMean(2),2);
			
			$p->setDimLabel("x","Time in seconds (in red, ". $this->getKeyFramesNumber($data)." keyframes for ".count($data->DataList)." frames) ");
			$p->setDimLabel("y","Bytes per frame (mean is $mean bps)");
			$p->setRange('y', 0, $yMax*1.2);

			/*
			$step=$yMax/10;
			$yTicks=array();
			for($y=0;$y<$yMax;$y=$y+$step) {
				$yTicks[]=round($y);
			}
			$p->setTics("y", "(".implode(", ",$yTicks) .")" ) ;
			*/

			$p->setRange('x', -0.1, $xMax*1.1);
			$p->setMargin('l',12);
			$p->setMargin('r',2);
			$p->setGrid();
			$p->setSizeof('point',2);
			$p->set2DLabel(1,"I frames");
			$p->set2DLabel(1,"graph", 0.85, 0.96 ,"","","tc lt 1");
			$p->set2DLabel(2,"P frames");
			$p->set2DLabel(2,"graph", 0.85, 0.92 ,"","","tc lt 2");
			$p->set2DLabel(3,"B frames");
			$p->set2DLabel(3,"graph", 0.85, 0.88 ,"","","tc lt 3");

			$p->plotData( $data,  'i lc rgb variable', '2:3:1 notitle' );

			//$p->set("autoscale");
			/*$p->setRange('y', 0, 5);
			$p->setSize( 0.6, 0.6 );
			*/

			$p->export("{$file_name}_{$data_type}_{$graph_type}.png");
			$p->debug();
			$p->close();

		}
		
		
	

		function get_data($array_values,$type) {
		//---------------------------------------
			
			$bitrate_cols=array();
			$CONSTANTS=get_defined_constants();
			
			if($type=="video"){
				$bitrate_cols["pict_type"]='I|B|P';
			}else {	
				$bitrate_cols["pkt_flag_key"]="_|K";
			}
			$bitrate_cols["pkt_pts"]=0;
			$bitrate_cols["size"]=0;
			
			foreach($bitrate_cols as $key=>$value) {
				$value=$array_values[$key];
				if($key=="pkt_flag_key") {
					$value=($value=='K') ? IFRAMES : PFRAMES;
				}else if($key=="pict_type") {
					$value=$CONSTANTS["{$value}FRAMES"];
				}else if($key=="size") { 
					$value=intVal($value);
				}else 
					$value=floatVal($value);
					
				$bitrate_cols[$key]=$value;
			}
			return array_values($bitrate_cols);
		}


	}
	
	
	function make_data_batch($video_name,$video_ext,$data='column.dat') {
	//---------------------------------------------
		$dir=FFPROBE_DIR;
		$video_input="$video_name.$video_ext";
		$bat=array(
			"rem from http://forums.creativecow.net/thread/291/71",
			call_exe($dir,"ffprobe",' -show_frames '.$video_input." > {$video_name}_frames.txt"),
			call_exe($dir,"cat","{$video_name}_frames.txt")." | ".call_exe($dir,"grep",' "codec_type\|size\|coded_picture_number\|pict_type" > raw.dat '),
			call_exe($dir,"paste",' -s -d \'\t\t\n\' raw.dat > fixed.dat'),
			call_exe($dir,"sed",'-e "s/size=//g" -e "s/coded_picture_number=//g" -e "s/size=//g" -e "s/pict_type=//g" -e "s/I/167116800/g" -e "s/P/65280/g" -e "s/B/255/g" fixed.dat > '.$data),
			call_exe($dir,"gnuplot",' plot.plt'),
			"pause"
		);
		$batch_output="$video_input.bat";
		$fp=file_put_contents($batch_output,implode("\n",$bat));
	}

?>