<?php

	/****************************************************************
	*@file parser.php
	*@package FFPROBE
	*@name  Toolbox to parse probe files generated with ffprobe 
	*@requires ini parser
	*@internal Don't use ffprobe of ffmpeg , use CVS version of ffprobe which has the -show_frames support!
	*@author Actarus
	*@requires ini.php to parse ini files
	*@version V1.0
	*@note 
	*	V2.0 (25.02.2011) DECODER and STREAMS added
	*	V1.0 (24.02.2011) Working release
	******************************************************************/


	if(!defined('INIPARSER_VERSION'))    die("FFProbe parser requires INIPARSER!");
	if(!class_exists("FFPROBE_Formats")) die("FFProbe parser requires the ini formater to parse correctly the -formats output");
	
	define("FFPROBE_PARSER_VERBOSE",0);
	
	
	class FFPROBE_Parser extends iniParser{
	 //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
	 
	   var $index;
	   var $type;
	   var $SectionName;
	   var $SectionNames;
	   var $codecs;
	 
	   //accept a filename or ini content as first parameter
		function FFPROBE_Parser($type,$FileName, $SafeFile = false){ //__construct
		//==================================================
		
			$this->type=$type;
			$SectionNames=$this->getSectionNames($type);
			$this->SectionNames=$SectionNames;
		
			$this->iniParser($FileName, $SafeFile);
			$this->SectionName=(is_array($SectionNames)) ? implode("|",$SectionNames) : $SectionNames;
			$this->index["$type"]=array("file"=>-1);
			$this->codecs=FFPROBE_get_codecs();
		}
		
		
		function getSectionNames($type) {
		//----------------------------
			$SectionNames=array(
				'codecs'  => 'DECODER',
				'files'   => 'FILE',
				'frames'  => 'FRAME',
				'packets' => 'PACKET',
				'streams' => 'STREAM',
				'tags' => 'TAGS',
				'formats' => explode(";",FFFORMATS_SECTIONS)
			);
			return $SectionNames["$type"];
		}
		
		function getCodecTypeFrom($codec_name) {
		//---------------------------------
			$codecs=$this->codecs;
			if(array_key_exists($codec_name,$codecs)) { //paranoïd security check
				$codec=$codecs[$codec_name];
				$codec_type=$codec[2];
				if($codec_type=="V") $codec_type="video";
				if($codec_type=="A") $codec_type="audio";
				if($codec_type=="S") $codec_type="subtitle";
				
			}else {
				die("FFPROBE_Parser encounters an alien codec named '<b>$codec_name</b>'. Can not determine codec's type. This is mainly due to your ffprobe version mismatches formats.ini created previously, use cleanup.bat and retry.");
			}
			return $codec_type;
		}
		
		 function addKey(&$Keys,$SectionName,$SectionIdx,$CurKey,$CurValue,$SaveInClass) {
		//--------------------------------------------------------------------------------	
			
			static $key_type;
			$type=$this->type;
			

			if(preg_match("/({$this->SectionName})/",$SectionName)) {
			
				//cast value to php types if wanted
				if($this->PhpCast) 
					$CurValue=$this->castValue($CurValue);
				
				if($this->verbose) {
					$SectionIdxShow=$SectionIdx;//abs($SectionIdx);
					$this->dump($CurValue," ->Item entry(<$CurKey>) for {$SectionName}[{$SectionIdxShow}]");
				}
				
				if(($SectionName == 'FRAME')||($SectionName == 'PACKET')||($SectionName == 'STREAM')||($SectionName == 'DECODER')) {
				
					if($SectionName == 'STREAM') { //for STREAM, 'codec_type' is unfortunately not the first entry of the section
						$isFirstKey=($CurKey=='codec_name');
						if($isFirstKey) {
							$codec_name=trim($CurValue);
							$key_type=$this->getCodecTypeFrom($codec_name);
						}
						//}
					}else if(($SectionName == 'FRAME')||($SectionName == 'PACKET')) { 
						//!NOTE Here we assume 'codec_type' is the first entry of the section 'FRAME' and 'PACKET'
						$isFirstKey=($CurKey=='codec_type');
						if($isFirstKey) $key_type=trim($CurValue);
					}else if($SectionName == 'DECODER') {
						$isFirstKey=($CurKey=='name');
						if($isFirstKey) {
							$codec_name=trim($CurValue);
							$key_type=$this->getCodecTypeFrom($codec_name);
						}
					}else{
						die("ffprobe::parser()".$SectionName.':'.$CurKey.':'.trim($CurValue));
					}
					
					if($isFirstKey) {
						if(!array_key_exists("$key_type",$this->index["$type"])){
							$this->index["$type"]["$key_type"]=0;
						}else {
							$this->index["$type"]["$key_type"]=$this->index["$type"]["$key_type"]+1;
						}
					}
					
					if( $SaveInClass ) {
						if(array_key_exists("$key_type",$this->index["$type"])) {
							if($this->index["$type"]["$key_type"]==0) {
								if($isFirstKey) {
									if(FFPROBE_PARSER_VERBOSE) echo "<br> NEW SINGLE {$SectionName}TYPE $key_type";
									$Keys[$SectionName][$key_type]=array("ffprobe_{$type}_number"=>$this->index["$type"]["file"],"codec_type"=>$key_type);
								}else {
									$Keys[$SectionName][$key_type][trim($CurKey)] = trim($CurValue);
								}
							}else {
								if(($this->index["$type"][$key_type]==1)&&array_key_exists("codec_type",$Keys[$SectionName][$key_type])) {
									if(FFPROBE_PARSER_VERBOSE) echo "<br> NEW REMIX FOR {$SectionName}TYPE $key_type";
									$Keys[$SectionName][$key_type]=array(0=>$Keys[$SectionName][$key_type]);
								}
								$frame_idx=$this->index["$type"]["$key_type"];
								if($isFirstKey) {
									if(FFPROBE_PARSER_VERBOSE) echo "<br> NEW MULTI {$SectionName}TYPE($key_type)";
									$Keys[$SectionName][$key_type][$frame_idx]["ffprobe_frame_number"]=$this->index["$type"]["file"];
								}
								$Keys[$SectionName][$key_type][$frame_idx][trim($CurKey)] = trim($CurValue);
							}
						}
					}else
						$NKeys[$SectionName][trim($CurKey)] = trim($CurValue);
						
				}else if($type=="formats"){ //see -formats in http://ffmpeg.org/ffmpeg-doc.html
					
					//cast value to php types if wanted
					if($this->PhpCast) 
						$CurValue=$this->castValue($CurValue);
					
					if($this->verbose) {
						$SectionIdxShow=abs($SectionIdx);
						$this->dump($CurValue," ->Item entry($CurKey) for {$SectionName}[{$SectionIdxShow}] ");
					}
					
					$curValue=trim($CurValue);
					$SectionNames=$this->SectionNames;
					
					switch($SectionName) {
						case $SectionNames[0]: //'File formats'
							$values=explode(":",$curValue);
							if(count($values)>=2) {
								$values[0]=($values[0]=="D") ? 1 : 0; //Decoding available 
								$values[1]=($values[1]=="E") ? 1 : 0; //Encoding available 
							}else { //failsafe
								$values=array(0,0); //Ghost file
							}
							$keys=array("Decoding"=>"D","Encoding"=>"E","Format"=>"F");
							$keys=array_values($keys);
							
							//$curValue=array_combine($keys,$values);
							$curValue=$values;
						break;
						case $SectionNames[1]: //codecs
							$keys=array("Decoding"=>"D","Encoding"=>"E","Type"=>"T","Slice"=>"S","Direct"=>"R","Truncated"=>"T","Codec"=>"C");
							$keys=array_values($keys);
							$keys_number=count($keys);
			
							$values=explode(":",$curValue);
							
							//!FIXME:first value is a ghost value generated from the regexp. It's not \n nor \s...get rid of
							array_shift($values);
							
							$values_number=count($values);
							
							$n=5; //security
							while(($n>0)&&($values_number>$keys_number)) {
								$value=array_pop($values);
								$values_number=count($values);
								$values[$values_number-1]=$values[$values_number-1].':'.$value;
								$n--;
							}
							
							$values[0]=($values[0]=="D") ? 1:0; //Decoding available 
							$values[1]=($values[1]=="E") ? 1:0; //Encoding available 
							
							/*
							if($values[2]=="V") $values[2]="video";
							if($values[2]=="A") $values[2]="audio";
							if($values[2]=="S") $values[2]="subtitle"; 
							*/
							
							$values[3]=($values[3]=="S") ? 1 : 0; //Codec supports slices 
							$values[4]=($values[4]=="D") ? 1 : 0; //Codec supports direct rendering 
							$values[5]=($values[5]=="T") ? 1 : 0; //Codec can handle input truncated at random locations instead of only at frame boundaries 
							
							if($keys_number==$values_number) {
								//$curValue=array_combine($keys,$values);
								$curValue=$values;
							}else {
								echo "<BR>WARNING: Houston we have a problem in addKey: codec key <b>$CurKey</b> has a set of values ($values_number) which differs from keys ($keys_number).";
							}
						break;
						case  $SectionNames[5]://'Notes':
						break;
						default://'Bitstream filters','Supported file protocols','Frame size, frame rate abbreviations'
							$curValue=explode(":",$curValue);
					}
					if($SectionName == $SectionNames[5]) {
						$CurKey=str_replace("text","",$CurKey);
					}else {
						$CurKey=trim($CurKey);
					}
					
					if( $SaveInClass ) {
						if($SectionIdx==-1) {
							$Keys[$SectionName][$CurKey] = $curValue;
						}else {
							$Keys[$SectionName][$SectionIdx][$CurKey] = $curValue;
						}
					}else
						$NKeys[$SectionName][$CurKey] = $curValue;
					
					
				}else { 
					$this->_addKey($Keys,$SectionName,$SectionIdx,$CurKey,$CurValue,$SaveInClass);
				}
			}else {
				echo "<br>WARNING : section $SectionName ignored";
			}
	   }


		function addKeysHolder(&$Keys,$SectionName) {
	   //---------------------------------------------
	   
			if(preg_match("/({$this->SectionName})/",$SectionName)) {
				$type=$this->type;
				if(array_key_exists($SectionName,$this->Keys)) {
					//read the frame type
					if($this->index["$type"]["file"]==0) {//array_key_exists("__SINGLE__",$this->Keys[$SectionName])) {
						if(FFPROBE_PARSER_VERBOSE) echo "<br> NEW REMIX FOR $SectionName";
						
						//unset($this->Keys[$SectionName]["__SINGLE__"]);
						if(($SectionName == 'FRAME')||($SectionName == 'PACKET')||($SectionName == 'STREAM')||($SectionName == 'DECODER')) {
							$key_type=array_keys($this->Keys[$SectionName]);
							$key_type=$key_type[0];
							$SectionType=$key_type;
						}else {
							echo "<pre>";
							var_dump($this->Keys[$SectionName]);
							echo "</pre>";
							$SectionType='';
							die("Stop with ($SectionName)");
						}
						
						$frame=array_values($this->Keys[$SectionName]);
						$this->Keys[$SectionName]=array("$SectionType"=>$frame);
					}

					$SectionIdx=$this->index["$type"]["file"]+1;
					$this->index["$type"]["file"]=$SectionIdx;
					if(FFPROBE_PARSER_VERBOSE) echo "<br> NEW MULTI $SectionName($SectionIdx)";
				}else {
					
					if(FFPROBE_PARSER_VERBOSE) echo "<br> NEW SINGLE $SectionName";
					$this->Keys[$SectionName]=array();//array("__SINGLE__"=>true);
					$SectionIdx=$this->index["$type"]["file"]+1;
					$this->index["$type"]["file"]=$SectionIdx;
					
				}
			} else {
				die("Invalid section name '<b>$SectionName</b>', should be one of {$this->SectionName}");
			}	
			return $SectionIdx;
	   }

	   
	}   

?>