<?php

	/****************************************************************
	*@file runner.php
	*@package FFPROBE
	*@name  Wrapper to run ffprobe 
	*@internal Don't use ffprobe of ffmpeg , use CVS version of ffprobe which has the -show_frames support!
	*@greetings I would like here to thanks Stefano Sabatini for his great tool, ffprobe
	*@author Actarus
	*@requires shell.php to run exe files
	*@version V1.0
	*@note 
	*	V1.0 (23.02.2011) Working release
	******************************************************************/


	if(!defined("SHELL_FACILITY")) die("FFProbe runner requires shell facility support!");
	
	function FFPROBE_get_shows($video_in='') {
	//-------------------------------------
		$shows=explode(",",FFPROBE_SHOWS);
		if($video_in) {
			$video=explode(".",$video_in);
			if(count($video)==2) {
				$video_name=$video[0];
				$video_ext=$video[1];
			}else {
				die("FFPROBE_get_shows can not read your video's file extension from your provided video file (<b>$video_in</b>).Crashing");
			}
			echo "Reading probe files of <font color=\"blue\">$video_name.$video_ext:</font>";
			$exts=array("success"=>0);
			foreach($shows as $show) {
				$ext=FFPROBE_get_show($show,$video_name,$video_ext);
				if($ext["success"]) $exts["success"]=$exts["success"]+1;
				$exts[$show]=$ext["return"];
			}
			$shows=$exts;
		}
		return $shows;
	}
	
	function FFPROBE_get_show($show,$video_name,$video_ext) {
	//------------------------------------------------
		
		$data_file="{$video_name}_$show.ini";
		$ret=FFPROBE_get_data_file($data_file);
		$success=$ret["success"];
		if($success) {
			$out=$ret["return"];
			$color="green";
			//process the generated file and adapt it for php
			$ini=new FFPROBE_Parser($show,$out);
		
			//$cfg=init_set("memory_limit","256M");
			$entries=$ini->parseIniWithSections();
			$return=$entries;
		}else {
			$color="red";
			$return=array();
		}
		echo "<font color=\"$color\">$show</font>";
		echo "]";
		
		$ret=array(
			"success"=>$success,
			"return"=>$return
		);
		return $ret;
	}
	
	function FFPROBE_get_data_file($data_file) {
	//---------------------------------------
		$success=(file_exists($data_file)&&is_readable($data_file));
		if($success) {
			$return=file_get_contents($data_file);
		}else {
			$return=null;
		}
		$ret=array(
			"success"=>$success,
			"return"=>$return
		);
		return $ret;
	}
	
	
	function FFPROBE_show_formats() {
	//-------------------------------
	
		$formats_file=FFPROBE_get_formats_file();
		$formats_data_file=FFPROBE_get_formats_data_file();
		
		$ret=FFPROBE_get_data_file($formats_data_file);
		
		if($ret["success"]) {
		
			$data=$ret["return"];
			$out=unserialize($data); 
		
		}else {

			if(!file_exists($formats_file)) {
				$ret=FFPROBE_generate("formats");
			}else {
				$ret=FFPROBE_get_data_file($formats_file);
			}
			
			if($ret["success"]) {
			
				$data=$ret["return"];
				
				//process the generated file and adapt it for php
				$ini=new FFPROBE_Parser("formats",$data);
			
				//$cfg=init_set("memory_limit","256M");
				$entries=$ini->parseIniWithSections();
				$out=$entries;
				
				//save decoding
				$data=serialize($out);
				file_put_contents($formats_data_file,$data);
				
			}else {
				$out=$ret;
			}
		}
		
		return $out;
	}
	
	
	function FFPROBE_show_all($video_in) {
	//-------------------------------------
	
		$video=explode(".",$video_in);
		
		if(count($video)==2) {
			$video_name=$video[0];
			$video_ext=$video[1];
		}else {
			die("FFPROBE_show_all can not read your video's file extension from your provided video file (<b>$video_in</b>).Crashing");
		}
			
		echo "Probing file <font color=\"blue\">$video_name.$video_ext:</font>";
		
		$shows=FFPROBE_get_shows();
		
		$datas=array();
		
		foreach($shows as $show) {
			echo "[";
			$ret=FFPROBE_generate($show,$video_name,$video_ext);
			$success=$ret["success"];
			$color=($success) ? "green" : "red";
			if($success) {
				$data=$ret["return"];
				
				$ini=new FFPROBE_Parser($show,$data);
			
				//$cfg=init_set("memory_limit","256M");
				//parses the data to get its array equivalent
				$data=$ini->parseIniWithSections();
				
				//save
				$datas=array_merge($datas,$data);
				
			}
			echo "<font color=\"$color\">$show</font>";
			echo "]";
		}
		
		$data_file="{$video_name}_showtime.dat";
		$data=serialize($datas);
		file_put_contents($data_file,$data);
		
		$ret["success"]=count($datas);
		$ret["return"]=$datas;
		return $ret;
	}
		
	
	//Main wrapper simply to run ffprobe to show $show for the video $video_input
	//Optionaly saves it to file $data_file
	function FFPROBE_run($show,$video_input='',$data_file='') {
	//---------------------------------------------------
		if($show =="formats") {
			$ret=run_exe(FFPROBE_DIR,"ffprobe"," -formats");
			if(!$ret["success"]) {
				$ret=run_exe(FFPROBE_DIR,"ffprobe"," -codecs"); //-bsfs -protocols -filters -pix_fmts
			}
		}else {
			$ret=run_exe(FFPROBE_DIR,"ffprobe"," -show_$show '".$video_input."'",$data_file); 
		}
		return $ret;
	}
	
	function FFPROBE_inspect() {
	//---------------------------
	
		$ret=run_exe_with_popen(FFPROBE_DIR,"ffprobe"," -h");
		$modules=array();
	
		if($ret["success"]) {
			$return=$ret["return"];

			
			$out=preg_split("/(Copyright|built on|configuration|Simple multimedia|usage)/",$return);
			
			$stamps=array_chunk(explode(" ",$out[2]), 5);
			
			$copyright=$out[1];
			$copyrights=explode("\n",$copyright);
			$copyright=trim($copyrights[0]);
			
			
			if(preg_match_all("/lib([\w]{2})(\w+)\s+([^\/]+)\/(.*)/", $return, $matches, PREG_SET_ORDER)) {
				foreach ($matches as $val) {
					$module=$val[0];
					$module_type=$val[1];
					$module_name=$val[2];
					$module_version=$val[3];
					if(!array_key_exists($module_type,$modules))$modules["$module_type"]=array();
					$modules["$module_type"]["$module_name"]=trim($module_version);
				}
			}
			
			$version=trim($out[0],', ');
			$date=trim(implode(" ",$stamps[0]),", ");
			$built=trim(str_replace("with","",implode(" ",$stamps[1])));
		
			
		} else {
			$version='FFprobe';
			$copyright="(c) 2007-2009 Stefano Sabatini";
			$date=filemtime("../ffprobe.php");
			$built='';
		}
		
		$ffprobe=array(
			"version"=>$version,
			"copyright"=>$copyright,
			"date"=>$date,
			"built"=>$built,
			"modules"=>$modules
		);
		
		return $ffprobe;
	}
	
	function FFPROBE_get_formats_file($cur_dir='',$safe_file=true) {
	//-----------------------------------------------------------
		$file=FFPROBE_FORMATS_FILE;
		if($safe_file) $file="$file.php";
		if(!$cur_dir)  $cur_dir=FFPROBE_get_current_dir();
		return $cur_dir.$file;
	}
	
	function FFPROBE_get_formats_data_file($cur_dir='',$safe_file=true) {
	//-----------------------------------------------------------
		$file=FFPROBE_FORMATS_DATA_FILE;
		if($safe_file) $file="$file.php";
		if(!$cur_dir)  $cur_dir=FFPROBE_get_current_dir();
		return $cur_dir.$file;
	}
	
	function FFPROBE_get_current_dir() {
	//-----------------------------------
		return fix_path(dirname(__file__))."/";
	}
	
	function FFPROBE_get_root_dir($cur_dir) {
	//--------------------------------------
		return resolve_path($cur_dir."../../")."/";
	}
	
	
	/**
	*Run ffprobe to show $show for the video '$video_name.$video_ext' and generate the ini file or a safe version .ini.php 
	*
	*@param String $show 
	*@param String video_name name of the video with its extension
	*@param String video_ext the video's extension
	*@param Boolean safe_file, if true produces a .ini.php instead of a classic .ini
	**/
	function FFPROBE_generate($show,$video_name='',$video_ext='',$safe_file=false) {
	//------------------------------------------------------------------------------------
	
		$cur_dir=FFPROBE_get_current_dir();
		$video_dir=FFPROBE_get_root_dir($cur_dir);
		
		if($video_name) {
			$video_input=$video_dir.$video_name.'.'.$video_ext;
			$data_file="{$video_name}_$show.ini";
			$ret=FFPROBE_run($show,$video_input,$data_file);
			
			if($ret["success"]) {
				$data=$ret["return"];
				if($data_file) {
					INI_save($data_file,$data,$safe_file);
					file_put_contents($data_file,$data);
				}
			} else {
				$data=null;
			}
			
			$ret["return"]=$data;
		}else { 
			$ret=FFPROBE_run("formats");
			
			if($ret["success"]) {
				//formats output is not ini compatible, convert it.
				$content=$ret["return"];
				$formats=new FFPROBE_Formats($content);
				$ini_content=$formats->makeINI();
				$data_file=FFPROBE_get_formats_file($cur_dir,false); //false because, .php will be appended on the next line
				INI_save($data_file,$ini_content,true);
				$ret["return"]=$ini_content;
			}
		}
		
		if(!$ret["success"]) {
			if(!$ret["err"]) $ret["err"]="The provided file cannot be found in dir $video_dir";
		}
		return $ret;
	}

?>