<?php

	/****************************************************************
	*@file parser.php
	*@name  Toolbox to parse ini  files
	*@package INI
	*@internal Fixed release from http://www.php.net/manual/en/function.parse-ini-file.php
	*@author Actarus
	*@version V5.0
	*@note 
	*	V5.0 (22.02.2011) Parse on ini content added
	*	V4.0 (05.11.2010) Remove comments was washing HTML entities / CSS properties, solved. PhpCast feature added
	*	V3.0 (03.11.2010) public, private keywords washed to make it php4 compatible
	*	V2.0 (27.10.2010) Fixed, dump and getIniEntries added, array reindexing was missing
	* 	V1.0 (22.10.2010) First release
	******************************************************************/

	if(!defined('INIPARSER_VERSION')) {
	
		define("INIPARSER_VERSION",5.0);
		define("INIPARSER_VERBOSE",0);
	
		class iniParser{
		   
		   //---private stuff---
			var $IniFile;
			var $SafeFile;
			var $ParseClasses;
			var $Keys;
			var $PhpCast;
			
			/***security**/
			var $lines_header; //real ini content starts at this line
			var $lines_footer; //real ini content ends this number of lines before
			var $verbose; 
			
			//---public stuff----
			var $rootSectionName;
			var $rootSectionEmptyShow;
			var $KeysWithoutSections;
			var $KeysWithSections;
			
		   //accept a filename or ini content as first parameter
			function iniParser($FileName, $SafeFile = false){ //__construct
			//==================================================
				$this->IniFile = $FileName;
				$this->SafeFile = $SafeFile;
				$this->lines_header=3;
				$this->lines_footer=3;
				$this->Keys=array();
				$this->rootSectionName='__ROOT__'; //Name of the virtual root, when parse with sections
				$this->rootSectionEmptyShow=false; //Hides root section if not root item
				$this->verbose=INIPARSER_VERBOSE; //For debug purposes only, if true, dumps all parsing and exit.
				$this->PhpCast=false; //If true, try to cast to php objects such as integers or booleans
			}
		   
			function dump($value,$tag="",$br=true) {
	 
				if($br) echo "<br>";
				if($tag) echo htmlentities($tag).':';
				if($value) echo htmlentities($value);
				
			}
			
			
			function dumpEntries($entries,$tag="",$halt=false) {
			
				if($tag) echo "<br><h4>$tag</h4><hr>";
				if($entries) {
					echo "<pre>";
					echo htmlentities(print_r($entries,true));
					echo "</pre>";
				}
				if($halt) exit();
			}
		   
			function getIniEntries() {
			
				$iniFile=$this->IniFile;
				
				if(preg_match("/([^\.]+)\.(\w{1,4})$/",$iniFile)) {
					$entries = file($iniFile);
				}else {
					$entries = explode("\n",$iniFile);
				}
				
				if($this->verbose) {
					$this->dumpEntries($entries,"Raw ini file entries");
				}
				
			//	if(!$SaveInClass) $NKeys=array();
			   
				if ( $this->SafeFile ){
						// Remove security, note array_values is required tohave a 0 indexed array key
						$from_start=$this->lines_header;
						$from_end=(-1*$this->lines_footer);
						$entries=array_slice($entries,$from_start,$from_end); 
						$entries=array_values($entries);
				}
				
				if($this->verbose) {
						$this->dumpEntries($entries,"Security Headers stripped");
				}
				
				return $entries;
			}
		   
		    //Safe remove in lazy mode: 
			//We will assume HTML entities &<name>; or CSS properties <name>:<value>px; has never spaces before ;
		    function _removeComments($matches) {
				//array_shift($matches);
				//$this->dumpEntries($matches,"Comments stripped");
				return ''; 
			}
		   
		   //Remove comments and track removal for debug purposes
		    function removeComments($CurValue) {
		   
			   //My PHP4/5 static call hack, only to make the callback $this->_removeComments($matches) possible!
				$callback_code='$THIS=new iniParser("[_STATIC_]");return $THIS->_removeComments($matches);';
				
				//Detect a line comment
				$CurValue = preg_replace_callback(
					'/(\s+);([^\;]+)/',
					create_function('$matches',$callback_code),
					$CurValue
				);
				
				return $CurValue;
			}
			
			function castValue($val) {

				if (preg_match('/^-?[0-9]$/i', $val)) { return intval($val); }
				else if (strtolower($val) === 'true') { return true; }
				else if (strtolower($val) === 'false') { return false; }
				else if (preg_match('/^"(.*)"$/i', $val, $m)) { return $m[1]; }
				else if (preg_match('/^\'(.*)\'$/i', $val, $m)) { return $m[1]; }
				
				return $val;
			}
		
		   
			function parseIni($SaveInClass = true){
		   
			   
			   $entries=$this->getIniEntries();
			   
			   $CountLines = count($entries);
			   
				$Counter=0;
				
				if($this->verbose) $this->dumpEntries("","Parsing $CountLines entries"); 
				
				while ( $Counter < $CountLines ){
				   
					$CurLine = $entries[$Counter];
					
					if($this->verbose) $this->dump($CurLine,"====Parsing Line($Counter)");
					 
				   if(trim($CurLine)!='') {
				   
						$CurLineSplit = explode("=", $CurLine);
					   
					   
					   if(count($CurLineSplit)>1) { //jump over sections
					   
							$CurKey = $CurLineSplit[0];
							$CurValue = $CurLineSplit[1];
							
							if($this->verbose) $this->dump($CurValue," ->Item entry($CurKey)");
							
							//remove comments 
							$CurValue=$this->removeComments($CurValue);
							
							//cast value to php types if wanted
							if($this->PhpCast) 
								$CurValue=$this->castValue($CurValue);
							
							if( $SaveInClass )
								$this->Keys[trim($CurKey)] = trim($CurValue);
							   
							else
								$NKeys[trim($CurKey)] = trim($CurValue);
						}else {
							if($this->verbose) $this->dump("possible section"," ->Ignore");
						}
						
				   } else if($this->verbose) $this->dump(" whitespace line"," ->Ignore");
				   
					$Counter++;
				}
				
			
			   
				if ( $SaveInClass )
					$out=$this->Keys;
				   
				else
					$out=$NKeys;
		   
				if($this->verbose) $this->dumpEntries($out,"Final WithoutSections",true);
		   
				return $out;
			}
		   
		   
		   function _addKey(&$Keys,$SectionName,$SectionIdx,$CurKey,$CurValue,$SaveInClass) {
				
				//cast value to php types if wanted
				if($this->PhpCast) 
					$CurValue=$this->castValue($CurValue);
				
				if($this->verbose) {
					$SectionIdxShow=abs($SectionIdx);
					$this->dump($CurValue," ->Item entry($CurKey) for {$SectionName}[{$SectionIdxShow}] ");
				}
				
				if( $SaveInClass ) {
					if($SectionIdx==-1) {
						$Keys[$SectionName][trim($CurKey)] = trim($CurValue);
					}else {
						$Keys[$SectionName][$SectionIdx][trim($CurKey)] = trim($CurValue);
					}
				}else
					$NKeys[$SectionName][trim($CurKey)] = trim($CurValue);
		   }
		   
		   function _addKeysHolder(&$Keys,$SectionName) {
		   
				if(array_key_exists($SectionName,$this->Keys)) {
					if(array_key_exists("__SINGLE__",$this->Keys[$SectionName])) {
						//echo "<br> NEW REMIX FOR MULTI";
						unset($this->Keys[$SectionName]["__SINGLE__"]);
						$this->Keys[$SectionName]=array(0=>$this->Keys[$SectionName]);
					}
					//echo "<br> NEW MULTI";
					$SectionIdx=count($this->Keys[$SectionName]);
				}else {
					//echo "<br> NEW SINGLE";
					$this->Keys[$SectionName]=array("__SINGLE__"=>true);
					$SectionIdx=-1;
				}
				return $SectionIdx;
		   }
		   
		   
			function parseIniWithSections($SaveInClass = true){
		   
			    $entries=$this->getIniEntries();
			  
			    $CountLines = count($entries);
				$Keys=&$this->Keys;
			
				if($this->verbose) $this->dumpEntries("","<a name=\"entries\">Parsing $CountLines entries</a>"); 
				
				
				$SectionName=$this->rootSectionName; //Virtual section for on top entries
				
				if( $SaveInClass ) {
					$Keys[$SectionName]=array();
				} else {
					$NKeys[$SectionName]=array();
				}
				
				$Counter=0;
				$SectionIdx=-1; //when positive we have redundant section's names
				
				
				while ( $Counter < $CountLines ){
				   
					$CurLine = $entries[$Counter];
				
					if($this->verbose) $this->dump($CurLine,"===Parsing Line($Counter)");
				   
				   
				   if(trim($CurLine)!='') {
				   
						$CurLineSplit = explode("=", $CurLine);
					   
					   if(count($CurLineSplit)>1) { //section's items
					   
							$CurKey = $CurLineSplit[0];
							$CurValue = $CurLineSplit[1];
							
							//remove comments
							$CurValue=$this->removeComments($CurValue);
							
							if(method_exists($this,"addKey")) { //custom method
								$this->addKey($Keys,$SectionName,$SectionIdx,$CurKey,$CurValue,$SaveInClass);
							}else {
								$this->_addKey($Keys,$SectionName,$SectionIdx,$CurKey,$CurValue,$SaveInClass);
							}
							
						}else if(preg_match('/\[([^\]]+)\]/', $CurLine, $matches)){ //Extract new section's name
							$SectionName=$matches[1]; 
							if($SectionName[0] != '/') { //Do not save XML-like closing sections
								if($this->verbose) $this->dump($SectionName," ->Section");
								if($SaveInClass ) {
								
									if(method_exists($this,"addKeysHolder")) { //custom method
										$SectionIdx=$this->addKeysHolder($Keys,$SectionName);
									}else {
										$SectionIdx=$this->_addKeysHolder($Keys,$SectionName);
									}
									
								}else {
									$NKeys[$Section]=array();
								}
							}else {
								if($this->verbose) $this->dump(str_replace("/","",$SectionName)."[$SectionIdx]"," ->Closing section");
							}
					   }
				   } else if($this->verbose) $this->dump("whitespace line"," ->Ignore");
					$Counter++;
				}
		   
				
			   
				if ( $SaveInClass )
					$Sections=$this->Keys;
				
				else
					$Sections=$NKeys;
		   
				if((!$this->rootSectionEmptyShow)&&(count($Sections[$this->rootSectionName])==0)) array_shift($Sections);
		   
		   
				if($this->verbose) $this->dumpEntries($Sections,"Final WithSections",true);
			
				
				return $Sections;
		   
			}
		   
		}

		
		function load_safe_ini($ini_file,$with_sections=false) {
		//-----------------------------------------------------
			//echo("Parse ini file $ini_file");
			$SafeIniParser = new iniParser($ini_file, true); // file: foo.inc.php, safefile.
			
			if($with_sections) {
				$keys = $SafeIniParser->parseIniWithSections(false);
			}else {
				$keys = $SafeIniParser->parseIni($false);
			}
			//var_dump($keys);
			//die("Root:$with_sections");
			return $keys;
		}
	}
?>