<?php

	/****************************************************************
	*@file shell.php
	*@name  Wrapper to run command shells in background 
	*@requires URL/toolbox.php to handle correctly paths
	*@version 3.0
	*@author Actarus
	*@note
	*   3.0  (20110226) - run_exe_with_popen added
	*	2.0  (20110223) - SHELL_FACILITY introduced
	*	1.0  (20110222) - First version 
	******************************************************************/


	if(!defined('URL_TOOLBOX')) die("Shell facility requires URL toolbox!");
		
	if(!defined("SHELL_FACILITY")) {	
			
		define("PHP5_ENGINE",version_compare(phpversion(), "5"));
		define("DEBUG_CMDLINE",0);
	 
	 
		//!NOTE try to detect your OS
		
		function is_windows(){
		//--------------------
			$PHP_OS=php_uname('s');
			return (strtoupper(substr($PHP_OS, 0, 3)) === 'WIN');
		}
		
		function is_mac() {
		//--------------
			$PHP_OS=php_uname('s');
			return (strtoupper(substr($PHP_OS, 0, 6)) === 'DARWIN'); //not tested
		}
		 
	 
		function call_cmd($dir,$cmd,$args='') {
		//-----------------------------------
			if($args) $args=''.$args;
			return ($dir) ? "\"$dir/$cmd\"$args" : "$cmd$args";
		}
	 
	 
		/**
		*Generate randomly an unique id
		*@note this is used to fight acrobat cache
		**/
		function rnunid() {
		//----------------
			return md5( uniqid() );  // 32 characters long
			//$unique = sha1( uniqid() );  // 40 characters long
		}
		
		function read_pipe($pipes,$n) {
		//-----------------------------
			if(PHP5_ENGINE) { 
				$content=stream_get_contents($pipes[$n]);
			}else { //Php4
				$content= "";
				while (($str = fgets($pipes[$n], 4096))) {
					$content.= "$str\n";
				}
			}
			return $content;
		}

		

		
		
		/**
		*@name  run_exe
		*@brief Wrapper to call $exe, as a shell command, in background
		*@access public 
		*@internal OS dependant
		*@param String dir absolute pathname the directory where the binary executable is located
		*@param String exe the name of the binary file to execute
		*@param String args the args to pass 
		*@return Array
		**/
		function run_exe($dir,$exe,$args) {
		//---------------------------------
		
			$descriptorspec = array(
				0 => array("pipe", "r"),  // // stdin 
				1 => array("pipe", "w"),  // stdout 
				2 => array("pipe", (is_windows()) ? "a" : "w") // stderr , "a" to avoid  STDOUT blocks infinitly on WINDOZE , thanks Luceo!
			);

			$cwd = '/tmp';
			$env = array('misc_options' => 'aeiou');
			$err='';
			$success=0;

			if(is_windows()) {
				$cmd="$exe.exe"; //For windows
			}else{
				$cmd="$exe"; //For linux and mac
			}
			
			$dircmd=resolve_path(fix_path(dirname(__file__).'/../../'.$dir));
			
			if(file_exists("$dircmd/$cmd")) {
			
				$cmdline=call_cmd($dircmd,$cmd,$args); //direct to ouptut	
			
				if(DEBUG_CMDLINE) echo htmlentities("$cmdline , $descriptorspec, $cwd, $env");

				if(PHP5_ENGINE) { // Php5
					$process = proc_open($cmdline, $descriptorspec, $pipes, $cwd, $env);
				}else { //Php4
					$process = proc_open($cmdline, $descriptorspec, $pipes);
				}

				if (is_resource($process)) {

					$err=read_pipe($pipes,2);
					$out=read_pipe($pipes,1);

					//Its important to close the pipes...
					fclose($pipes[0]);
					fclose($pipes[1]);
					fclose($pipes[2]);
					
					//...before proc_close call to avoid  dead locks 
					$return_value = proc_close($process);
					
				}else {
					$err="No more resource to execute the command";
				}
				
			}else {
				$err="The <b>$cmd</b> binary was not found in <b>'$dircmd'</b><ol>";
				$err.="<li>check the exe name</li>";
				$err.="<li>check the directory name</li></ol>" ;
				$out='';
				$return_value=1;
			}
			
			$ret=array("success"=> ($return_value==0) ,"err"=>$err,"return"=>$out );
		
			return $ret;
		}
		
		//same command as before but with popen
		function run_exe_with_popen($dir,$exe,$args) {
		//-----------------------------
		
			if(is_windows()) {
				$cmd="$exe.exe"; //For windows
			}else{
				$cmd="$exe"; //For linux and mac
			}
			
			$dircmd=resolve_path(fix_path(dirname(__file__).'/../../'.$dir));
			
			if(file_exists("$dircmd/$cmd")) {
			
				$cmdline=call_cmd($dircmd,$cmd,$args); //direct to ouptut	
			
				if(DEBUG_CMDLINE) echo htmlentities("$cmdline , $descriptorspec, $cwd, $env");
				
				//redirect for errors
				$process = popen($cmdline.' 2>&1', 'r');
				$content= "";
				while (($str = fgets($process, 4096))) {
					$content.= "$str\n";
				}
				pclose($process);
				$err='';
				$out=$content;
			}else {
				$err="The <b>$cmd</b> binary was not found in <b>'$dircmd'</b><ol>";
				$err.="<li>check the exe name</li>";
				$err.="<li>check the directory name</li></ol>" ;
				$out='';
				$process=false;
			}
			
			$ret=array("success"=> !(is_bool($process)&&($process==false)) ,"err"=>$err,"return"=>$out );
		
			return $ret;
		}
		
		
		
		
		define("SHELL_FACILITY",1);
	}
?>