<?php

	/****************************************************************
	*@file ffprobe.php
	*@package FFPROBE
	*@name Main script for Php ffprobe 
	*@author Actarus
	*@version V4.0
	*@note 
	*	V4.0 (28.02.2011) graph support using phpgnuplot 
	*	V3.0 (27.02.2011) ffprobeinfo() roxx with ffmpeg info and credits  directly read from ffprobe!
	*	V2.0 (25.02.2011) Nice wrapping ffprobeini() and ffprobeinfo() added
	*	V1.0 (24.02.2011) Working release
	******************************************************************/

	include('gui.php');
	include('parsers.php');
	include('shell.php');
	include('plot.php');
	
	include("INI/safe.php");       //Extra for security
	include('FFProbe/formats.php');//ini adapter for output -formats
	include('FFProbe/parser.php');
	include('FFProbe/runner.php');
	include('FFProbe/graph.php');

	//=========== Configuration ==================== 
	define("FFPROBE_DIR",'..\\ffmpeg-r26402-ffprobe with gnuplot\\bin\\');//please adjust the path if needed.
	define("FFPROBE_SHOWS",'codecs,files,frames,packets,streams,tags'); //with formats, it's the total exhibition
	define("FFPROBE_SHOWS_SECTIONS",'DECODER,FILE,FRAME,PACKET,STREAM,TAGS');//
	define("FFPROBE_FORMATS_FILE","formats.ini");//Beware because if safe file, .php in appended automaticaly.
	define("FFPROBE_FORMATS_DATA_FILE","formats.inc");//Holds the array equivalent of FFPROBE_FORMATS_FILE
	define("FFPROBE_FORMATS_DATA_SECTIONS","File formats,Codecs,Bitstream filters,Supported file protocols,Frame size and frame rate abbreviations,Notes");
	define("FFPROBE_CODEC_TYPES","video,audio,subtitle");
	define("FFPROBE_CODEC_PROCESSING","decoding,encoding,<>,slicing,showing,truncating"); //<> is codec_type , we declare it.
	define("FFPROBE_VERSION","4.0");
	define("FFPROBE_DATE","28.02.2011");
	define("FFPROBE_AUTHOR","Actarus");
	define("FFPROBE_GREETINGS","Stefano Sabatini, Liu Yi ,Michael Rampe");
	define("FFPROBE_LICENSE","This program makes use of FFprobe's from Stefano Sabatini , based on FFmpeg's engine both LGPL.<br>Feel free to use PHP4FFprobe without license nightmare issues as it follows the GPL's stream as well =)<br>");
	//================================================
	
	$FFPROBE_formats=null; //Array that will hold ffmpeg/ffprobe's formats
	
	/*
	* Creates the FFPROBE_FORMATS_FILE, decodes it as an array and saves it into $FFPROBE_formats
	*
	* Two goals achieved from this:
	* -The installation of ffprobe is checked because we use it. Normaly we have to do this once.
	* -As we got now the codecs table, bridge between the codec's type and the codec's name, required by streams, is now built and available.
	**/
	function FFPROBE_init() {
	//----------------------
		return FFPROBE_get_php_formats();
	}
	
	/**
	*Dumps the ffprobe info or the video info data
	*
	*@note this function should not be called directly, use ffprobeinfo instead
	*@param String category 'shows' or 'formats'
	*@param array data a data according the category context 
	**/
	function FFPROBE_info($category,$data) {
	//-------------------------------------
		GUI_info('PHP4FFProbe '.FFPROBE_VERSION,$category,$data);
	}
	
	
	//#######################################################################
	//## Here the two main public functions to centralize php4ffprobe powaa.
	//## and use it without a pain. These are for dumping
	//########################################################################
		
	/**
	*Main function to initialize FFProbe
	*
	*@param Int timeout in seconds;Adjust this in case of big video files
	**/
	function ffprobeinit($timeout=60){
	//----------------------------------
		set_time_limit($timeout);
		return FFPROBE_init();
	}
	
	/**
	* Main fonction to probe a video/audio file and display its data properties
	*
	*@param String|Array video a  video data or video file
	*@return array
	**/
	function ffprobeinfo($video='') {
	//--------------------------------
		global $FFPROBE_formats;
		if($video=='') {
			$video_data=$FFPROBE_formats;
			FFPROBE_info("formats",$video_data);
		}else {
			if(is_string($video)) {
				$videofile=$video;
				$ret=FFPROBE_show_all($videofile);
				$success=$ret["success"];
				$video_data=$ret["return"];
			}else if(is_array($video)) {
				$video_data=$video;
				$video="video's data";
				$success=true;
			} else 
				die("<br><u>ffprobeinfo() error</u>: first param should be either a video_data or video file");
				
			if($success) {	
				FFPROBE_info('shows',$video_data);
			} else 	{
				echo("<br><br><u>ffprobeinfo() failure</u>: Cannot generate data file from $video, reason was:<br>");
				echo($ret["err"]);
				if(!preg_match("/provided/",$ret["err"])) {
					echo("<font color=\"blue\"><u>Change your ffprobe directory config :</u></font><br>");
					echo("FFPROBE_DIR is set to <font color=\"violet\">".FFPROBE_DIR."</font> in <u>lib/ffprobe.php</u> you need to adjust it to point to your ffprobe directory");
				}
			}
		}
		return $video_data;
	}
	


	//#######################################################################
	//## From here, data is a formats data array 
	//## For all the following functions of = 'video' , 'audio'  or 'subtitle'
	//########################################################################
	
	
	function FFPROBE_count_decoding_formats() {
	//---------------------------------------
		return count(FFPROBE_get_decoding_formats());
	}
	
	function FFPROBE_get_decoding_formats() {
	//---------------------------------------
		return FFPROBE_get_processing_formats('decoding');
	}
	
	function FFPROBE_count_encoding_formats() {
	//---------------------------------------
		return count(FFPROBE_get_encoding_formats());
	}
	
	function FFPROBE_get_encoding_formats() {
	//---------------------------------------
		return FFPROBE_get_processing_formats('encoding');
	}
	
	function FFPROBE_count_formats() {
	//--------------------------------
		return count(FFPROBE_get("File formats"));
	}
	
	
	function FFPROBE_get_processing_formats($process) {
	//---------------------------------------
		$formats_list=FFPROBE_get("File formats");
		
		if($process=="decoding") $keep_filter='$format_props[0]';
		if($process=="encoding") $keep_filter='$format_props[1]';
		
		$formats=array_filter($formats_list,create_function(
			'$format_props',
			'$keep=('.$keep_filter.');return $keep;'
		));
		/*	
		echo "<pre>";
		var_dump($formats);
		echo "</pre>";
		
		die("Stop");*/
		return $formats;
	}
	
	
	function FFPROBE_get_formats() {
	//-----------------------------
		return FFPROBE_get("File formats");
	}
	
	function FFPROBE_get_codecs() {
	//-----------------------------
		return FFPROBE_get("Codecs");
	}
	
	function FFPROBE_get_filters() {
	//-----------------------------
		return FFPROBE_get("Bitstream filters");
	}
	
	function FFPROBE_get_protocols() {
	//--------------------------------
		return FFPROBE_get("Supported file protocols");
	}
	
	function FFPROBE_get_framesize_and_rate_abbreviations() {
	//-------------------------------------------------------
		return FFPROBE_get("Frame size, frame rate abbreviations");
	}
	
	function FFPROBE_get_notes() {
	//----------------------------
		return FFPROBE_get("Notes");
	}
	
	function FFPROBE_count_codecs($of,$data,$silent=true) {
	//-----------------------------------------------------
		return count(FFPROBE_codecs($of,$data,$silent));
	}
	
	
	//Get the $of=(video|audio|subtitle) codec list
	function FFPROBE_codecs($of,$data,$silent=true) {
	//-----------------------------------------------
		return FFPROBE_codecs_for_processing($of,null,$data,$silent);
	}

	//decoding
	function FFPROBE_count_codecs_for_decoding($of,$data,$silent=true) {
	//--------------------------------------------------------------
		return  count(FFPROBE_codecs_for_decoding($of,$data,$silent)); 
	}
	

	function FFPROBE_codecs_for_decoding($of,$data,$silent=true) {
	//--------------------------------------------------------------
		return  FFPROBE_codecs_for_processing($of,'decoding',$data,$silent);
	}

	//encoding
	function FFPROBE_count_codecs_for_encoding($of,$data,$silent=true) {
	//--------------------------------------------------------------
		return  count(FFPROBE_codecs_for_encoding($of,$data,$silent)); 
	}
	

	function FFPROBE_codecs_for_encoding($of,$data,$silent=true) {
	//--------------------------------------------------------------
		return  FFPROBE_codecs_for_processing($of,'encoding',$data,$silent);
	}
	
	//Process core
	function FFPROBE_codecs_for_processing($of,$process,$data,$silent=true) {
	//--------------------------------------------------------------------
		$codecs=array();
		if(is_array($data)) {
			if(array_key_exists("Codecs",$data)){
				$codecs_list=array_values($data["Codecs"][1]);
				$codec_types=str_replace(",","|",FFPROBE_CODEC_TYPES);
				if(preg_match("/($codec_types)/",$of)) {
					$codec_type_code=strtoupper($of[0]);
					
					$filter_exps=array(
						'$codec_value[2]'=>$codec_type_code
					);
					
					if(!is_null($process)) {
						$process_list=array();
						if(is_string($process)) $process_list[]=$process;
						if(is_array($process))  $process_list=$process;
						
						foreach($process_list as $process) {
							switch($process) {
								case "decoding": $filter_exps['$codec_value[0]']=1;
								break;
								case "encoding": $filter_exps['$codec_value[1]']=1;
								break;
								case "slicing": $filter_exps['$codec_value[3]']=1;
								break;
								case "showing": $filter_exps['$codec_value[4]']=1;
								break;
								case "truncating": $filter_exps['$codec_value[5]']=1;
								break;
								default:
									echo("<br> WARNING unknown codec process '$process'");
							}
						}
					}
					
					$keep_filters=array();
					foreach($filter_exps as $filter_expname => $filter_expvalue) {
						$keep_filters[]="($filter_expname=='$filter_expvalue')";
					}
					$keep_filter=implode("&&",$keep_filters);
					
					$codecs=array_filter($codecs_list,create_function(
						'$codec_value',
						'$keep=('.$keep_filter.');return $keep;'
					));
					
					//if(!$silent) echo "<br>FFPROBE_codecs() can not find the $of codec list, sorry.";
				}else if(!$silent) echo "<br>FFPROBE_codecs() claims your codec type ($of) is invalid should be on of the list (".FFPROBE_CODEC_TYPES.")";
			} else if(!$silent) echo "<br>FFPROBE_codecs() requires -formats support your ffprobe is lacking.";
		}else if(!$silent) echo "<br>FFPROBE_codecs() claims you data is invalid, should be an array";
		return $codecs;
	}
	
	//#######################################################################
	//## From here, data is a shows data array
	//## For all the following functions of = 'video' , 'audio'  or 'subtitle'
	//########################################################################
	
	function FFPROBE_count_frames($of,$data,$silent=true) {
	//-----------------------------------------------------
		return count(FFPROBE_frames($of,$data,$silent));
	}
	
	function FFPROBE_frames($of,$data,$silent=true) {
	//-----------------------------------------------------
		$frames=array();
		if(is_array($data)) {
			if(array_key_exists("FRAME",$data)){
				$frame=$data["FRAME"];
				if(array_key_exists($of,$frame)) {
					$frames=$frame[$of];
				}
			} else if(!$silent) echo "Sorry, this requires -show-frames support your ffprobe is lacking.";
		}else if(!$silent) echo "Invalid data, should be an array";
		return $frames;
	}
	
	function FFPROBE_count_keyframes($of,$data,$silent=true) {
	//---------------------------------------------------------
		return count(FFPROBE_keyframes($of,$data,$silent));
	}
	
	function FFPROBE_keyframes($of,$data,$silent=true) {
	//-------------------------------------------------
		$keyframes=array();
		if(is_array($data)) {
			if(array_key_exists("FRAME",$data)){
				$frame=$data["FRAME"];
				if(array_key_exists($of,$frame)) {
					$frames=$frame[$of];
					$count=count($frames);
					if($count) {
						$keyframes=array_filter($frames,create_function(
							'$frame',
							'return ($frame["pkt_flag_key"]=="K");'
						));
					}
				}
			} else if(!$silent) echo "Sorry, this requires -show-frames support your ffprobe is lacking.";
		}else if(!$silent) echo "Invalid data, should be an array";
		return $keyframes;
	}
	
	
?>