<?php

/**
 * A PHP Interface to GNU Plot
 *
 * Copyright (C) 2006 Liu Yi (Eric) <eric.liu.yi@gmail.com>
 *
 * Website : http://celeste.cn
 *
 * The main purpose of this program is to facilitate plotting with data
 * generated from program. It is not a *Complete* interface to GNU Plot.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 * @note
 *	2.0 (28.02.2011) Imported
 **/

//include "shell.php";

define("PHP_GNUPLOT_VERSION","2.0");

 //Debug support by Actarus
define("DEBUG_GNUPLOT",0);
define("DEBUG_GNUPLOT_FILE",0);


// change this if u need
//$GNUPLOT = 'C:\gnuplot\bin\pgnuplot.exe'; // for Windows
//$GNUPLOT = 'C:\Program Files\ffmpeg-r26402\bin\gnuplot.exe';
$GNUPLOT = 'C:\gnuplot\binary\pgnuplot.exe';

// $GNUPLOT = "gnuplot";  // for linux

$tempDir = 'C:\temp'; // somewhere we can store the temporary data files

// DONT change the code below if you dont know what you are doing
$IDCounter = 0;

/**
 * PGData is a general data unit for series plotting
 * 
 * It can either be created from an external data file ( in this case, it is simply a wrapper to the file )
 * or created with an empty list of entries ( You can append entries to the list ).
 * 
 * Before it is plotted, if it is created in the 2nd way, it must have its data dumped into a file.
 *
 **/
 
class PGData {
//@@@@@@@@@@

	// private variables
	var $filename; // Name of the data file. Can be explicitly specified or automatically generated
	var $DataList; // This is only useful when $filename is not specified or for debug purposes
	var $legend; // Title of the data you want to see on the graph
	var $index; //Index of the last min or max seeked with getMin or getMax;
	
	function PGData($legend = '') {
	//===========================
		$this->legend = $legend;
		$this->filename='';
	}

	/**
	 * static method to initialize a data object from an external data file
	 * the object is just a wrapper to the file
	 **/
	function createFromFile($filename, $legend = '')  {
	//-------------------------------------------------
		$Data = new PGData($legend);
		if (!file_exists($filename) || !is_readable($filename)) {
			print "Error: $filename is not a readable datafile!\n";
			return NULL;
		}
		$Data->filename = $filename;
		$Data->parse();
		return $Data;
	}

	function parse() {
	//--------------------
		// Lit une page web dans un tableau.
		$lines = file($this->filename);

		// Affiche toutes les lignes du tableau comme code HTML, avec les numros de ligne
		foreach ($lines as $line_num => $line) {
			$this->DataList[]=preg_split("/\s+/",$line);
		}
	}
	
	function dump($data=null) {
	//----------------
		$data_entries=(is_null($data)) ? $this->DataList : $data ;
		echo "<pre>";
		print_r($data_entries);
		echo "</pre>";
	}
	
	function getCol($n) {
	//-------------------
		$col=array();
		$data_entries=$this->DataList;
		$nMax=count($data_entries[0]);
		if($n<$nMax) {
			foreach( $data_entries as $id => $entry ) {
				$col[]=$entry[$n];
			}
		}
		return $col;
	}
	
	
	function getMax($col) {
	//---------------------
		return max($this->getCol($col));
	}
	
	function getMean($col) {
	//---------------------
		$values=$this->getCol($col);
		$values_count=count($values);
		return ($values_count) ? array_sum($values)/$values_count : 0;
	}
	
	function getMin($col) {
	//-----------------
		return min($this->getCol($col));
	}
	
	function getStats($col) {
	//----------------------
		return array_count_values($this->getCol($col));
	}
	
	
	function changeLegend( $legend ) { $this->legend = $legend; }
	
	function addDataEntry( $entry ){
		$filename=$this->filename;
		if (!$filename) $this->DataList[] = $entry;
			else print "Error: Cannot add an entry into file content [ $filename ] !\n";
		
	}
	
	function dumpIntoFile( $filename='' ) {
		
		if ($this->filename) { print "Error: Data file exists [ $this->filename ] !\n"; return; }
		global $tempDir, $IDCounter;
		if (!$filename) {
			// generate a file name
			$filename = $tempDir.'/data_'. ( $IDCounter++ ) .'.txt';
			global $toRemove;
			$toRemove[] = $filename;
		}
		$fp = fopen($filename, 'wt'); //force as text else it will be considered as binary!
		
		$data_entries=$this->DataList;
		
		foreach( $data_entries as $entry ) {
			fwrite($fp, implode("\t", $entry)."\n");
		}
		
		fclose($fp);
		$this->filename = $filename; // no longer changeable
	}
}

/**
 * The main class to communicate with GNU Plot
 * It opens a pipe to a GNU Plot process
 * 
 * You can guess the idea from the names of the functions
 * Some of the parameters are explained in the comments
 **/

class GNUPlot {
//@@@@@@@@@@@@

	// private variables
	var $ph = NULL;
	var $cd = array(); //Commands dump
	var $toRemove;
	var $plot;
	var $splot;
	var $labels;
	var $file; //Plt file for debug purpose only
	
	var $image; //output image attributes
	var $plotareas; 
	var $pl; //id of the last plotarea 
	
	function GNUPlot($file='') {
	//=================
		global $GNUPLOT;
		if(USE_GRAPHPITE) {
			$graph=new Image_Graph(400, 300);
			$graph->hideLogo();
			$this->ph = $graph;
		}else {
			if(!file_exists($GNUPLOT)||!is_readable($GNUPLOT)) die("<br><font color=\"red\">Gnuplot pipe executable can not be reached in this directory (<font color=\"violet\">$GNUPLOT</font>), <br>change the path in <font color=\"violet\">lib/plot/PHP_GNUPlot.php</font> or (re)install it.</font></br>");
			$this->ph = popen($GNUPLOT, 'w');
		}
		$this->toRemove = array();
		$this->labels=array();
		$this->plot = 'plot';
		$this->splot = 'splot';
		$this->file=$file;
		$this->image=array("type"=>'',"file"=>'');
		$this->plotareas=array();
		$this->pl=-1;
	}

	//usefull to see what commands are generated
	function debug() {
	//---------------------
		global $GNUPLOT;
		$file=$this->file;
		if(DEBUG_GNUPLOT) {
			if($file&&DEBUG_GNUPLOT_FILE) {
				$contents=implode("\n",$this->cd);
				file_put_contents($file,$contents);
				echo "<br>Gnuplot's file saved as <b>$file</b>";
				$contents="\"$GNUPLOT\" $file\npause";
				$file=str_replace(".plt",".bat",$file);
				file_put_contents($file,$contents);
				echo "<br>Gnuplot's batch file saved as <b>$file</b>";
			}else {
				echo "<pre>";
				print_r($this->cd);
				echo "</pre>";
			}
		}
	}
	
	
	// You can tell from the name of the function.
	
	function draw2DLine($x1,$y1, $x2, $y2){
	//-------------------------------------
	   if(USE_GRAPHPITE) {
			$this->set("arrow","2D",$x1,$y1,$x2,$y2,"nohead");
	   }else {
			$this->set("arrow","from $x1,$y1 to $x2,$y2 nohead" );
	   }
	}
	
	function draw3DLine($x1,$y1,$z1, $x2, $y2,$z2){
	//---------------------------------------------
	  if(USE_GRAPHPITE) {
			$this->set("arrow","3D",$x1,$y1,$z1,$x2,$y2,z2,"nohead");
	   }else {
			$this->set("arrow","from $x1,$y1,$z1 to $x2,$y2,$z2 nohead" );
		}
	}

	function draw2DArrow($x1,$y1, $x2, $y2){
	//--------------------------------------
		if(USE_GRAPHPITE) {
			$this->set("arrow","2D",$x1,$y1,$x2,$y2,'head');
		}else {
			$this->set("arrow","from $x1,$y1 to $x2,$y2 head" );
		}
	}
	
	function draw3DArrow($x1,$y1,$z1, $x2, $y2,$z2){
	//-----------------------------------------------
		if(USE_GRAPHPITE) {
			$this->set("arrow","3D",$x1,$y1,$z1,$x2,$y2,$z2,'head');
		}else {
			$this->set("arrow","from $x1,$y1,$z1 to $x2,$y2,$z2 head" );
		}
	}
	
	/*core function for 2D and 3D label.This function is polymorphic:
		First pass (if $tag)
		$tag, $text, $pre='',$justify='', $extra='',$x, $y 
		$tag, $text, $pre='',$justify='', $extra='',$x, $y, $z
		Second pass (if $tag)
		$tag, $text=$pre , '',$justify='', $extra='',$x, $y
		$tag, $text=$pre , '',$justify='', $extra='',$x, $y,$z
	**/
	function setxDLabel($tag,$text,$pre='',$justify='',$extra='',$x=null,$y=null,$z=null) {
	//-----------------------------------------------------------------------------------------
	
		if($tag != '') $tag=" $tag ";
		
		$at='';
		if(!is_null($x)) $at=" at";
		$x=(is_null($x)) ? '':" $x";
		$y=(is_null($y)) ? '':",$y";
		$z=(is_null($z)) ? '':",$z";
		if($justify) $justify=" $justify";
		if($extra) $extra=" $extra";
		
		
		if(array_key_exists("$tag",$this->labels)) {
			$labeltext='';
			$at="at";
			$pre=$text;
		}else {	
			$labeltext="\"". $text ."\"";
			$this->labels["$tag"]=$text;
		}
		if($pre) $pre=" $pre"; 
		$this->set("label","$tag$labeltext$at$pre$x$y$z$justify$extra");
	}

	function set2DLabel(/*[$tag,]$labeltext, $x=null, $y=null, $pre='', $justify='', $extra=''*/ ) {
	//---------------------------------------------------------------------------
		// $justify =  {left | center | right}
		// $pre = { first|second|graph|screen }
		
		$args=func_get_args();
		
		if(!is_int(func_get_arg(0))) {  //reproduces the shift we have when tag is here as first arg
			array_unshift($args,"");
		}
		
		$n=count($args);
		$tag=$args[0];
		
		//mandatory
		$labeltext=$args[1];
		
		//optional if tag
		$x=($n>2)? $args[2] : null;
		$y=($n>3) ? $args[3] : null;
		
		//always optional
		
		$pre=($n>4) ? $args[4] : '';
		$justify=($n>5) ? $args[5] : '';
		$extra=($n>6) ? $args[6]: '';
		
		$this->setxDLabel($tag,$labeltext,$pre,$justify,$extra,$x,$y);
	}
	
	function set3DLabel(/*[$tag,]$labeltext, $x=null, $y=null, $z=null, $pre='', $justify='',$extra='' */) {
	//-------------------------------------------------------------------------------------------------------
		// $justify =  {left | center | right}
		// $pre = { first|second|graph|screen }
		
			
		$args=func_get_args();
		
		if(!is_int(func_get_arg(0))) {  //reproduce the shift we have when tag is here as first arg
			array_unshift($args,"");
		}
		
		$n=count($args);
		$tag=$args[0];
		
		//mandatory
		$labeltext=$args[1];
		
		//optional if tag
		$x=($n>2)? $args[2] : null;
		$y=($n>3) ? $args[3] : null;
		$z=($n>4) ? $args[4] : null;
		
		//always optional
		$pre=($n>5) ? $args[5] : '';
		$justify=($n>6) ? $args[6] : '';
		$extra=($n>7) ? $args[7]: '';
		
		$this->setxDLabel($tag,$labeltext,$pre,$justify,$extra,$x,$y,$z);
	}
	
	function setGrid() {
	//-------------------
		$this->set("grid");
	}
	
	function setSizeof($of,$value) {
	//---------------------------
		$this->set("{$of}size",$value);
	}
	
	function setMargin($which,$value) {
	//---------------------------------
		// $which = (r | l | t | b)
		$this->set("{$which}margin",$value);
	}
	
	function setRange( $dimension, $min, $max, $extra='' ) {
	//------------------------------------------------------
		// $dimension = x, y, z ......
		if (!$dimension) $dimension = 'x';
		if(USE_GRAPHPITE) {
			$this->set("range",$dimension,$min,$max,$extra);
		}else {
			$this->set("${dimension}range","[$min:$max] $extra");
		}
	}

	// low level set command
	function set( $toSet,$args='') {
	//------------------------------
		if(USE_GRAPHPITE) {
			$args=func_get_args();
			array_unshift($args,"set");
			call_user_func_array(array($this, "exe"),$args);
		} else {
			$this->exe("set",$toSet,$args);
		}
	}

	function setTitle( $title, $extra='' ) {
	//--------------------------------------
		if(USE_GRAPHPITE) {
			$this->set("title",$title,$extra);
		} else {
			$this->set("title","\"$title\" $extra");
		}
	}

	// Set label for each axis
	function setDimLabel($dimension,$text,$offset=0,$font=array("name"=>'',"size"=>0),$textcolor='',$line_type='',$enhanced=false,$degrees=0) {
	//----------------------------------------------------------------------------------------------------------------------------------------

		// $dimension = x, y, z ......
		if((func_num_args()==3)&&is_string(func_get_arg(2))) {
			$ALabel=array(
				"text"=>$text
			);
			$extras=func_get_arg(2);
		} else {
			$ALabel=array(
				"text"=>$text,
				"offset"=>$offset,
				"font"=>$font,
				"textcolor"=>$textcolor,
				"linetype"=>$line_type,
				"enhanced"=>$enhanced,
				"degrees"=>$degrees
			);
			
			$extras='';
			if($ALabel["offset"]) $extras.=" {$ALabe["offset"]}";
			if($ALabel["font"]["name"]) {
				$extras.=" {$ALabel["font"]["name"]}";
				if($ALabel["font"]["size"]) 
					 $extras.=",{$ALabel["font"]["size"]}";
			}
			if($ALabel["textcolor"]) {
				 $extras.=" {$ALabel["textcolor"]}";
			}
			if($ALabel["linetype"]) {
				 $extras.=" {$ALabel["textcolor"]}";
			}
			
			if($ALabel["enhanced"]) {
				$extras.=" enhanced";
			}else {
				$extras.=" noenhanced";
			}
			
			if($ALabel["degrees"]) {
				$args.=" rotate by {$ALabel["degrees"]}";
			}
		}
		
		$args="\"{$ALabel["text"]}\"$extras";
		
		if(USE_GRAPHPITE) {
			$this->set("label",$dimension,$text,$offset,$font,$textcolor,$line_type,$enhanced,$degrees);
		}else {
			$this->set("{$dimension}label",$args);
		}
	}
	

	function setTics( $dimension, $option ) {
	//---------------------------------------
		// $dimension = x, y, z ......
		if(USE_GRAPHPITE) {
			$this->set("tics",$dimension,$option);
		}else {
			$this->set("${dimension}tics","$option" );
		}
	}
	
	function setSize( $x, $y, $extra='' ) {
	//-----------------------------------
		// $extra = {{no}square | ratio <r> | noratio}
		if(USE_GRAPHPITE) {
			$this->set("size",$extra,$x,$y);
		}else {
			$this->set("size","$extra $x,$y");	
		}
	}

	function plotData(  &$PGData, $method, $using, $axis='', $extra='' ) {
	//--------------------------------------------------------------------
		/**
		 * This function is for 2D plotting
		 *
		 * $method is `lines`, `points`, `linespoints`, `impulses`, `dots`, `steps`, `fsteps`, 
		 *			  `histeps`, errorbars, `xerrorbars`, `yerrorbars`, `xyerrorbars`, errorlines, 
		 *			  `xerrorlines`, `yerrorlines`, `xyerrorlines`, `boxes`, `filledcurves`, 
		 *			  `boxerrorbars`, `boxxyerrorbars`, `financebars`, `candlesticks`, `vectors` or pm3d 
		 *
		 * $using is an expression controlling which data columns to use and how to use:
		 * 			Example : $using = " 1:2 " means plotting column 2 against column 1
		 *					  $using = " ($1):($2/2)  " means use half of the value of column 2 to plot against column 1
		 *			You can introduce in more than 2 or 3 columns to enable styles like errorbars
		 **/
		
		$plot = $this->plot;
		if (!$PGData->filename) $PGData->dumpIntoFile();
		if (!$PGData->filename) { print "Error: Empty dataset!\n"; return; }

		$fn = $PGData->filename;
		$title = $PGData->legend;

		if ($axis) $axis = " axis $axes ";
		
		if(preg_match("/notitle/",$using)) {
			$title='';
		}else {
			$title=" title \"$title\""; 
		}
		if(USE_GRAPHPITE) {
			$this->exe($plot,$fn,$using,$title,$method,$axis,$extra);
		}else {
			$this->exe($plot,''," '$fn' using $using$title with $method  $axis $extra");
		}
		
		//print "$plot '$fn' using $using title \"$title\" with $method  $axis $extra\n";
		$this->plot = 'replot';
		
	}

	function splotData( &$PGData, $method, $using, $extra = '' ) {
	//------------------------------------------------------------
	
		/**
		 * This function is for 3D plotting
		 *
		 */
		$splot = $this->splot;
		if (!$PGData->filename) $PGData->dumpIntoFile();
		if (!$PGData->filename) { print "Error: Empty dataset!\n"; return; }

		$fn = $PGData->filename;
		$title = $PGData->legend;

		if(USE_GRAPHPITE) {
			$this->exe($splot,$fn,$using,$title,$method,$extra);
		}else {
			$this->exe($splot," '$fn' using $using title \"$title\" with $method $extra\n");
		}
		$this->splot = 'replot';
		
	}
	
	
	// $width,$height since gnuplot 4.5
	function export( $pic_filename/*,$width,$height*/ ) {
	//-----------------------------
		/**
		 * export to $pic_filename
		 * the file ext can be png, ps or eps
		 */
		 
		 // you can add more options here to support other formats
		 $pic_exts=array(
			"eps"=>"postscript",
			"png"=>"png",
			"ps"=>"postscript"
		 );
		 
		 $pic_exts_mask=implode("|",array_keys($pic_exts));
		 
		if (preg_match("/\.($pic_exts_mask)$/", $pic_filename,$ext)){
			$pic_ext=$ext[1];
		}else { //failsafe
			$pic_ext="png";
			$pic_filename.=".$pic_ext";
		}
		
		$pic_set=$pic_exts["$pic_ext"];
		
		$args=func_get_args();
		
		if(count($args)>2) {
			$width=$args[1];
			$height=$args[2];
			$pic_set.=" size $width, $height";
		}
		
		$this->set("term",$pic_set);
		
		if(USE_GRAPHPITE) {
			$this->set("output",$pic_filename);
		} else {
			$this->set("output","\"$pic_filename\"");
		}
		$this->exe("replot");
	}

	function reset() {
		/**
		 * It is not a good idea to use this function in your program
		 */
		$this->exe( "reset" );
		$this->plot = 'plot';
		$this->splot = 'splot';
	}
	
	function close() {
	//---------------
		if(USE_GRAPHPITE) {
			$this->ph->saveAs(str_replace(".","g.",$this->image["file"]));
			$this->ph->Done();
		}else {
			flush($this->ph);
			pclose($this->ph);
			sleep(2);//Let gnu plot flush image files to the disk
			//sleep(count($this->toRemove) ); // allow gnu plot to finish so that we can safely remove the data files
			//foreach($this->toRemove as $filename) unlink($filename);
		}
	}

	function compare() {
	//-----------------
		$image_file=$this->image["file"];
		$image_file_gnuplot=$image_file;
		$image_file_graphpite=str_replace(".","g.",$image_file);
		echo "<table><tr><td><img src=\"$image_file_gnuplot\"></td><td><img src=\"$image_file_graphpite\"></td></tr></table>";
	}
	
	// command [,args]
	// 'set', command ,args
	function exe() {
	//--------------
		$func_args=func_get_args();
		$nb_args=count($func_args);
		
		if($nb_args>0) {
			$command=array_shift($func_args); 
		}else
			die("<br>exe() requires at least a command name");
		
		$args=array();
		if($nb_args>1) {
			if($command == 'set') {
				if($nb_args>2) {
					$set_command=array_shift($func_args);
					$args=$func_args;
				}else
					die("<br>set() requires at least a command name with args");
			}else {
				$args=$func_args;
			}
		}
		
		$command_args=implode(' ',$args);
		if($command_args)  $command_args=' '.$command_args;
		if($command == 'set') {	
			$cmd=$command.' '.$set_command.$command_args;
		}else {
			$cmd=$command.$command_args;
		}
		$this->cd[]=$cmd; //keep a track yo know what happens in the background
		$cmd=$cmd."\n";

		if(USE_GRAPHPITE) {//======SUPPORT IS BROKEN, I HAVE NO TIME FOR IT =============

			$Graph=&$this->ph;
			switch($command) {
				case "set":
					switch($set_command) {
						case 'arrow': // set arrow from 0,0 to 1,1 nohead 
						break;
						case 'title': //set title "2D Test" 
						
						break;
						case 'label': //set label "2D Label" at 1,1 
							//$graph
						break;
						case 'term':
							$this->image["type"]=$args[0];
						break;
						case 'output':
							$this->image["file"]=$args[0];
						break;
						case 'range'://set yrange [0:5] 
						
						break;
						case 'size'://set size 0.6,0.6 
							
						break;
						default:
							echo "<br>$command ($set_command) : $cmd";
					}
				break;
				case 'replot':
					//replot : replot 'demo2D.txt' using ($1/20):($2*2) title "data set II" with linespoints
					//replot : replot 'demo2D.txt' using ($1/20):($2) title "replot II" with boxes 
				case 'plot':
					//plot : plot 'C:\temp/data_0.txt' using 1:($2) title "test Data" with lines 
					$this->pl=$this->pl+1;
					
					if($this->pl==0) {
					    // add a TrueType font
						$Arial =& $Graph->addFont(new Image_Graph_Font_TTF("arial.ttf"));
						// set the font size to 15 pixels
						$Arial->setSize(11);
					}
					
					$this->plotareas[]=&new Image_Graph_Plotarea();
					
					// create a Y grid
					$Grid =& $this->plotareas[$this->pl]->addGridY(new Image_Graph_Grid_Lines()); 
					$Grid->setLineColor(IMAGE_GRAPH_LIGHTGRAY);
					
					// create some colors
					$Red=new Image_Graph_Color(255, 0, 0);
					$Yellow=new Image_Graph_Color(255, 255, 0);
					$Black=new Image_Graph_Color(0, 0, 0);
					$White=new Image_Graph_Color(255, 255, 255);
					$Lightgray=new Image_Graph_Color(0xee, 0xee, 0xee);
					$Blue=new Image_Graph_Color(0, 0, 127);
					$Transparent=new Image_Graph_Color(10, 10, 10);
					
					// and use black border
					$Graph->setBorderColor(IMAGE_GRAPH_BLACK);
					
					$Graph->add(
						new Image_Graph_Layout_Vertical(
							$Title=new Image_Graph_Title("**TITLE**", $Arial),
							new Image_Graph_Layout_Horizontal(
								new Image_Graph_Title(" Label Y", $Arial),
								new Image_Graph_Layout_Vertical(
									$this->plotareas[$this->pl],
									new Image_Graph_Layout_Vertical(
										new Image_Graph_Title("Label X", $Arial),
										$Legend_BarAndLine = new Image_Graph_Legend(),
										10
									),
									80
								),
								10
							),
							5
						)
					); 
					
					
		
					
					//$Legend_BarAndLine->setPlotArea($this->plotareas[$this->pl]);
					
					
					
					# add a title using the created font
					$Title->setText("");
 					$Graph->add(new Image_Graph_Title("GraPHPite - Sample", $Arial));
					
					
					$Legend_BarAndLine->setPlotArea($this->plotareas[$this->pl]);
					
					//$this->plotareas[$this->pl]->add(new Image_Graph_Legend());
					
					$Dataset =& new Image_Graph_Dataset_Trivial();
					$Dataset->addPoint(1, 100);
					$Dataset->addPoint(2, 41);
					$Dataset->addPoint(3, 78);
					$Dataset->addPoint(4, 12);
					
					//$GridY =& $this->plotareas[$this->pl]->addGridY(new Image_Graph_Grid_Bars()); 
					
					// create the plot as bar chart using the dataset
					$Plot =& $this->plotareas[$this->pl]->addPlot(new Image_Graph_Plot_Bar($Dataset));
					$Plot->setTitle("Try     ");
				
				
				
					
					 // add coins-icon as marker
					$Plot->setMarker(new Image_Graph_Marker_Icon("./images/coins.png"));
					
					$SomeColor=new Image_Graph_Color(0, 0, 255);
					$Graph->addColor($SomeColor, "SomeColor");
					$LineStyle =& new Image_Graph_Line_Solid($SomeColor);
					$Plot->setLineStyle($LineStyle);
					
					$Marker=new Image_Graph_Marker_Value(IMAGE_GRAPH_PCT_Y_MAX);
					//$Marker=new Image_Graph_Marker_Cross(IMAGE_GRAPH_PCT_Y_MAX);
					//$Marker=new Image_Graph_Marker_Circle(IMAGE_GRAPH_PCT_Y_MAX);
					//$Marker=new Image_Graph_Marker_Plus(IMAGE_GRAPH_PCT_Y_MAX);
					
				 
					$Plot->add($Marker, "Marker");
					
				
					// create a pin-point marker type
					$PointingMarker =new Image_Graph_Marker_Pointing_Angular(20, $Marker);
					$Plot->setMarker($PointingMarker);
				
				/*
					$Marker->setFillStyle($Yellow);
 					$Marker->setBorderStyle($Black);

					// and use the marker on the 1st plot
					$Plot->setMarker($Marker); 
*/

					/*
					$this->plotareas[]=&new Image_Graph_Plotarea();
					$this->pl++;
					$Graph->add($this->plotareas[$this->pl]); 
					*/
					
					// create a vertical gradient fill using red and yellow, ie bottom of
					// graph will be yellow and the "higher" the value the more red it will
					// be, ie a "fire" effect
					
					$FillArray =new Image_Graph_Fill_Gradient(IMAGE_GRAPH_GRAD_VERTICAL, IMAGE_GRAPH_RED, IMAGE_GRAPH_YELLOW, 100);
					
				/*	$FillArray =& new Image_Graph_Fill_Array();
					$FillArray->add($BLUE, 'Test1');
					$FillArray->add($RED, 'Test2');
					$FillArray->add($YELLOW, 'Test3');		
					*/
 					$Plot->setFillStyle($FillArray);
					
					//new Image_Graph_Plot_Smoothed_Area(
					
/*
					$FillArray =& new Image_Graph_Fill_Array();
					$Plot->setFillStyle($FillArray);
					$FillArray->add(new Image_Graph_Fill_Gradient(IMAGE_GRAPH_GRAD_RADIAL, IMAGE_GRAPH_GREEN, IMAGE_GRAPH_WHITE, 200));
					$FillArray->add(new Image_Graph_Fill_Gradient(IMAGE_GRAPH_GRAD_RADIAL, IMAGE_GRAPH_BLUE, IMAGE_GRAPH_WHITE, 200));
					$FillArray->add(new Image_Graph_Fill_Gradient(IMAGE_GRAPH_GRAD_RADIAL, IMAGE_GRAPH_YELLOW, IMAGE_GRAPH_WHITE, 200));
					$FillArray->add(new Image_Graph_Fill_Gradient(IMAGE_GRAPH_GRAD_RADIAL, IMAGE_GRAPH_RED, IMAGE_GRAPH_WHITE, 200));
					$FillArray->add(new Image_Graph_Fill_Gradient(IMAGE_GRAPH_GRAD_RADIAL, IMAGE_GRAPH_ORANGE, IMAGE_GRAPH_WHITE, 200));
	*/				
					
					
					
					
			
					$Plot2 =& $this->plotareas[$this->pl]->addPlot(new Image_Graph_Plot_Line($Dataset), "Plot");
					//$Plot2->setLineStyle($LineStyle);
				
					// create a Y data value marker
 					
					
					
					$AxisX =& $this->plotareas[$this->pl]->getAxis(IMAGE_GRAPH_AXIS_X);
					
					/*$XArray = array(
						"7 Aug 2004",
						"8 Aug 2004",
						"9 Aug 2004"
					);
					
					$AxisX->setDataPreprocessor(new ArrayData($XArray));
					*/
			
					
					$AxisY =& $this->plotareas[$this->pl]->getAxis(IMAGE_GRAPH_AXIS_Y);
					$AxisY->forceMaximum(200);
					
					
					// create formatting label data based on an array

					$ArrayData =& new Image_Graph_DataPreprocessor_Array(
						array(
							1=>"A Point",
							2=>"Another point"
						)
					);

					// use the data label array on the X axis

					$AxisX->setDataPreprocessor($ArrayData);
					
					//$AxisY->showArrow();
					//$Graph->Thumbnail(100, 100);
					
				break;
				default:
				echo "<br>$command : $cmd";
			}
		} else {
			fwrite($this->ph, $cmd);
		}
	}
	
}


// in case you are using old PHP. just in case
if (!function_exists('file_put_contents')) {
	function file_put_contents( $fn, $contents ) {
		$fp = fopen($fn, 'wt');
		fwrite($fp, $contents);
		fclose($fp);
	}
}
